# coding: utf-8
import fnmatch
import locale
import os
import unittest
import zipfile

from tionix_vdi_client.i18n import switch_language

try:
    from tionix_client.functional import DetailedTestRunner
except ImportError:
    DetailedTestRunner = None

FUNC_TESTS_PATTERN = 'func_test_vdi_client_*'
EN = 'en_US.UTF-8'

language, _encoding = locale.getdefaultlocale()
ENV_LANGUAGE_CODE = language or EN


def discover_tests_in_zip(path):
    zip_path, file_ext, _tests_path = path.partition('.zip')

    with zipfile.ZipFile(zip_path + file_ext) as zip_file:
        func_tests = [
            f_path for f_path in zip_file.namelist() if
            fnmatch.fnmatch(f_path, '*' + FUNC_TESTS_PATTERN)
        ]

    suite = unittest.TestSuite()
    for test in func_tests:
        # make importable module from path
        test = test.split('.')[0].replace('/', '.')
        suite.addTest(unittest.TestLoader().loadTestsFromName(test))

    return suite


def run_func_tests(runner=DetailedTestRunner, debug=False):
    test_result = None

    if runner:
        tests_dir = os.path.dirname(__file__)

        if runner is DetailedTestRunner:
            switch_language(ENV_LANGUAGE_CODE)

        if '.zip' in tests_dir:
            # unittest.TestLoader().discover fails to
            # work with files inside zip archive
            suite = discover_tests_in_zip(tests_dir)
        else:
            suite = unittest.TestLoader().discover(
                start_dir=tests_dir,
                pattern=FUNC_TESTS_PATTERN)

        test_result = runner(debug=debug).run(suite)
        if runner is DetailedTestRunner:
            test_result = test_result.to_dict()

    return test_result


if __name__ == '__main__':
    run_func_tests()
