# coding=utf-8

from StringIO import StringIO
import traceback

import unittest
from unittest.runner import _WritelnDecorator

from tionix_vdi_client.i18n import ugettext as _


class TestResultStatus(object):
    OK = _(u'OK')
    ERROR = _(u'ERROR')
    FAIL = _(u'FAIL')
    SKIPPED = _(u'SKIPPED')


class SkipTest(unittest.case.SkipTest):
    def __init__(self, reason=None):
        # convert unicode string to bytestring to prevent
        # UnicodeEncodeError in unittest.TestCase
        if isinstance(reason, unicode):
            reason = reason.encode('utf-8')
        super(SkipTest, self).__init__(reason)


class ExtendedTextTestResult(unittest.TestResult):
    def __init__(self, stream, descriptions, verbosity, debug):
        super(ExtendedTextTestResult, self).__init__(
            stream, descriptions, verbosity)
        self.success = []
        self._start_time = 0
        self.time = 0
        self.tests_results = []
        self.debug = debug
        self._original_skip_test = unittest.case.SkipTest

    def _exc_info_to_string(self, err, test):
        exctype, value, tb = err
        if exctype is AssertionError:
            message = value.message
            if isinstance(message, str):
                message = message.decode('utf8')
        else:
            if not self.debug:
                tb = None
            message = u''.join(
                traceback.format_exception(exctype, value, tb))
        return message

    def startTestRun(self):
        # patch original unittest SkipTest exception to allow
        # non-ascii symbols in skip test reason
        unittest.case.SkipTest = SkipTest

    def stopTestRun(self):
        # revert startTestRun patching
        unittest.case.SkipTest = self._original_skip_test

    def addSuccess(self, test):
        self.success.append(test)
        self.process_test(test, status=TestResultStatus.OK)

    def addError(self, test, err):
        super(ExtendedTextTestResult, self).addError(test, err)
        if 'ModuleImportFailure' in test.__class__.__name__:
            exctype, value, tb = err
            value = value.message.splitlines()[0]
            err = exctype, value, tb
        self.process_test(
            test,
            status=TestResultStatus.ERROR,
            details=self._exc_info_to_string(err, test))

    def addFailure(self, test, err):
        super(ExtendedTextTestResult, self).addFailure(test, err)
        self.process_test(
            test,
            status=TestResultStatus.FAIL,
            details=self._exc_info_to_string(err, test))

    def addSkip(self, test, reason):
        super(ExtendedTextTestResult, self).addSkip(test, reason)
        self.process_test(
            test, status=TestResultStatus.SKIPPED, details=reason)

    def process_test(self, test, status, details=u''):
        """Process test and update results dict"""
        test_name = unicode(test).split()[0].replace('_', ' ')
        if isinstance(details, str):
            details = details.decode('utf-8')
        self.tests_results.append((test_name, status, details))


class ExtendedTextTestRunner(unittest.TextTestRunner):
    """A test runner class with extended resultclass"""

    def __init__(self, debug=False, *args, **kwargs):
        super(ExtendedTextTestRunner, self).__init__(*args, **kwargs)
        self.stream = _WritelnDecorator(StringIO())
        self.resultclass = ExtendedTextTestResult
        self.debug = debug

    def _makeResult(self):
        return self.resultclass(self.stream, self.descriptions, self.verbosity,
                                self.debug)
